function [hBondMatrix] = parseHBond_hbm (fileName, doPlot)
% function parses a file created by the g_hbond function of gromacs via the
% -hbm option 
%
% How to parse the h-bonds: The -num option can be parsed with the function
% "parseXvgFile.m" and the -hbm with the function "parseHBond_hbm.m".
% The -hbn option with the function "parseHBond_hbn.". Further information
% on the atom type and coordinates etc can be obtained via "parsePdb.m"
%
%
% myPath = '/data/housemartin/knapp/projects/1mi5_sims/172_pdbFiles/172/finalSimulations/wildtype/';
% hBondMatrix = parseHBond_hbm ([myPath, '7Y.hbondMtx.MHCvsPeptide.xvg.xpm'], true);
% hBondNum = parseXvgFile ([myPath, '7Y.hbondNum.MHCvsPeptide.xvg']);
% plotXvgFile(hBondNum); 
%
% myPath = '/run/media/knapp/Seagate Expansion Drive/1mi5_172_100ns_sims/1A/';
% hBondMatrix = parseHBond_hbm ([myPath, '1A.hbondMtx.TCRvsPeptide.xvg.xpm'], true);
% hBondNum = parseXvgFile ([myPath, '1A.hbondNdx.TCRvsPeptide.ndx']);
% plotXvgFile(hBondNum);
% myPath = '/run/media/knapp/Seagate Expansion Drive/1mi5_172_100ns_sims/1A/';
% hBondNdx = parseNdx_from_hBond ([myPath, '1A.hbondNdx.TCRvsPeptide.ndx']);
%
% [hBondMatrix] = parseHBond_hbm ('/data/housemartin/knapp/matlabWorkspace/util/parsing/ascii/g5_h-bond/test.hbm.xpm', true);
%
% B. Knapp 2013-08-20

if nargin == 1
    doPlot = true;
end

[data, ~] = readFile (fileName);

[headerInfo, lineIter] = parseHeader (data);
% fprintf('header extracted ...\n');
hBondMatrix.headerInfo = headerInfo;
[mtxRaw] = parseHBondMatrix (data, lineIter);
% fprintf('raw data extracted ...\n');
mtx = convertMtxToValues (mtxRaw, headerInfo);
% fprintf('converted to values ...\n');

hBondMatrix.mtx = mtx;


if doPlot  
    plotXpm_from_hBondMatrix (hBondMatrix, '');
end
% fprintf('done.\n');



function [mtx] = convertMtxToValues (mtxRaw, headerInfo)
mtx = zeros(size(mtxRaw));

mapArr = [];

if strcmp(headerInfo.charCodes.values(1), 'None')
    mapArr(headerInfo.charCodes.charsAsNumber(1)) = 0; % we write 0 for no hbond. Do not confuse this with the previous o written for existance!
else
    error ('expected "None" but found "%s". Is your input a vaild xpm file created by the -hbm option of g_hbond?', headerInfo.charCodes.values(1));
end

if strcmp(headerInfo.charCodes.values(2), 'Present')
    mapArr(headerInfo.charCodes.charsAsNumber(2)) = 1; % we write 1 for a present hbond
else
    error ('Expected "Present" but found "%s". Is your input a vaild xpm file created by the -bhm option of g_hbond?', headerInfo.charCodes.values(2));
end


% substitute values
for xIter = 1:size(mtxRaw,1)
    for yIter = 1:size(mtxRaw,2)
        mtx(xIter, yIter) = mapArr(mtxRaw (xIter, yIter));
    end
end



function [mtx] = parseHBondMatrix (data, lineIter)
% eg
% " o o                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                             ",
% "                       o                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         ",
% "oooooooooooooooooo ooooooooooooooooooooooooooooooooooooooooooooooooooooo  ooooo oooooooooooooooooo ooooooooooooooooo oooooooooooooooooooooooo ooooooooooooooooo ooooooooooooooooo ooooooooooooooooooooooooooo o oooooooooooooo oo ooooooooooooooooooo ooooooooooooooooo oooooooooo o ooooo oooooo ooo oooooo oooo oo ooooooooooooooooooooo oooooooooooooooooooo ooooooooooooooooooooooooooooooooooo oooooooooooooooooooooooooooo ooooooooooooooooooooooooo ooooooooooooooooooooo ooooo oooooooooooooooooooooooooo  oooo oooo o oo o     oooooooooo ooo o  o                     o    o  o    oooooooo oooooooo ooo o oooo o ooooooo oooooooooooo oooo                                    o    o                                                                 o   o oo  oooo                                o      o       oo o",

line = data{lineIter};
mtx = zeros(length(data)-lineIter, length(line(2:end-2)));
ctr = 1;
while lineIter < length(data)
    idx = strfind(line, '"');
    mtx(ctr, :) = line(idx(1)+1:idx(2)-1); % implicit cast to numeric!
    ctr = ctr + 1;
    lineIter = lineIter + 1;
    line = data{lineIter};
end
idx = strfind(line, '"'); % also do it for the last line
mtx(ctr, :) = line(idx(1)+1:idx(2)-1);



function [headerInfo, lineIter] = parseHeader (data)

yAxisScalingFound = false;
headerInfo.charCodes = {};
headerInfo.charCodes.chars = {};
headerInfo.charCodes.charsAsNumber = [];
headerInfo.charCodes.colorcodes = {};
headerInfo.charCodes.values = [];
headerInfo.xAxis = [];
headerInfo.yAxis = [];

lineIter = 1; line = data{lineIter};

while ~strcmp(line(1), '"') ||  ~yAxisScalingFound % only the header
    
    if length(line) >= 8 && strcmp (line(1:9), '/* title:')
        % /* title:   "Hydrogen Bond Existence Map" */
        idx = strfind (line, '"');
        headerInfo.title = line(idx(1)+1:idx(2)-1);
    elseif length(line) >= 10 && strcmp (line(1:10), '/* x-label')
        % /* x-label: "Time (ps)" */
        idx = strfind (line, '"');
        headerInfo.xLabel = line(idx(1)+1:idx(2)-1);        
    elseif length(line) >= 10 && strcmp (line(1:10), '/* y-label')
        % /* y-label: "Hydrogen Bond Index" */
        idx = strfind (line, '"');
        headerInfo.yLabel = line(idx(1)+1:idx(2)-1);    
    elseif length(line) >= 10 && strcmp (line(1:7), '/* type')
        % /* type:    "Discrete" */
        idx = strfind (line, '"');
        headerInfo.type = line(idx(1)+1:idx(2)-1);   
    elseif length(line) >= 10 && strcmp (line(1:9), '/* legend')    
        % /* legend:  "Hydrogen Bonds" */
        idx = strfind (line, '"');
        headerInfo.legend = line(idx(1)+1:idx(2)-1);   
    elseif length(line) >= 10 && strcmp(line(1), '"') && strcmp (line(3:4), '  ')
        % "   c #FFFFFF " /* "None" */,
        % "o  c #FF0000 " /* "Present" */,
        headerInfo.charCodes.chars{end+1} = line(2);
        headerInfo.charCodes.charsAsNumber(end+1) = line(2);
        headerInfo.charCodes.colorcodes{end+1} = line(7:13);
        idx = strfind (line, '"');
        value = line(idx(3)+1:idx(4)-1);
        headerInfo.charCodes.values{end+1} = value;
        
    elseif (length(line) >= 10) && strcmp (line(4:10), 'x-axis:')
        vals = textscan(line(11:end), '%f'); % skip the first 10 chars ("/* x-axis:"), start at pos 11
        headerInfo.xAxis = [headerInfo.xAxis; vals{1}];
    elseif (length(line) >= 10) && strcmp (line(4:10), 'y-axis:')
        yAxisScalingFound = true;
        vals = textscan(line(11:end), '%f'); % skip the first 10 chars ("/* y-axis:"), start at pos 11
        headerInfo.yAxis = [headerInfo.yAxis; vals{1}];
    end
    
    lineIter = lineIter+1; line = data{lineIter};
end



